static const char *shader_color_vp_src =
#ifdef DX_GLES
   "#version 100\n"
   "\n"
#endif // DX_GLES
   "uniform mat4 u_transform;\n"
   "\n"
#ifdef DX_GLES
   "attribute highp vec4 a_vertex;\n"
   "attribute mediump vec4 a_color;\n"
#else
   "attribute vec4 a_vertex;\n"
   "attribute vec4 a_color;\n"
#endif
   "\n"
   "varying vec4 v_color;\n"
   "\n"
   "void main() {\n"
   "   gl_Position = u_transform * a_vertex;\n"
   "   v_color = a_color;\n"
   "}\n"
   ;

static const char *shader_color_fp_src = 
#ifdef DX_GLES
   "precision mediump float;  // set default precision\n"
   "\n"
#endif
   "varying vec4 v_color;\n"
   "\n"
   "void main() {\n"
   "   gl_FragColor = v_color;\n"
   "}\n  "
   ;


// ---------------------------------------------------------------------------- shader_texture_2d
static const char *shader_texture_2d_vp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "//precision mediump float;  // set default precision\n"
#endif // DX_GLES
   "\n"
   "uniform mat4 u_transform;\n"
   "\n"
#ifdef DX_GLES
   "attribute highp   vec4 a_vertex;\n"
   "attribute mediump vec4 a_color;\n"
   "attribute highp   vec4 a_texcoord;\n"
#else
   "attribute vec4 a_vertex;\n"
   "attribute vec4 a_color;\n"
   "attribute vec4 a_texcoord;\n"
#endif
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_Position = u_transform * a_vertex;\n"
   "   v_color = a_color;\n"
   "   v_texcoord = a_texcoord;\n"
   "}\n  "
   ;

static const char *shader_texture_2d_fp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "precision mediump float;  // set default precision\n"
   "\n"
#endif // DX_GLES
   "uniform sampler2D u_sampler;\n"
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_FragColor = texture2D(u_sampler, v_texcoord.xy) * v_color;\n"
   "}\n  "
   ; 


// ---------------------------------------------------------------------------- shader_texture_3d
static const char *shader_texture_3d_vp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "//precision mediump float;  // set default precision\n"
   "\n"
#endif // DX_GLES
   "uniform mat4 u_transform;\n"
   "\n"
#ifdef DX_GLES
   "attribute highp   vec4 a_vertex;\n"
   "attribute mediump vec4 a_color;\n"
   "attribute highp   vec4 a_texcoord;\n"
#else
   "attribute vec4 a_vertex;\n"
   "attribute vec4 a_color;\n"
   "attribute vec4 a_texcoord;\n"
#endif
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_Position = u_transform * a_vertex;\n"
   "   v_color = a_color;\n"
   "   v_texcoord = a_texcoord;\n"
   "}\n  "
   ;

static const char *shader_texture_3d_fp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "precision mediump float;  // set default precision\n"
   "\n"
#endif // DX_GLES
   "uniform sampler3D u_sampler;\n"
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_FragColor = texture3D(u_sampler, v_texcoord.xyz) * v_color;\n"
   "}\n  "
   ; 


// ---------------------------------------------------------------------------- shader_texture_cube_map
static const char *shader_texture_cube_map_vp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "//precision mediump float;  // set default precision\n"
   "\n"
#endif // DX_GLES
   "uniform mat4 u_transform;\n"
   "\n"
#ifdef DX_GLES
   "attribute highp   vec4 a_vertex;\n"
   "attribute mediump vec4 a_color;\n"
   "attribute highp   vec4 a_texcoord;\n"
#else
   "attribute vec4 a_vertex;\n"
   "attribute vec4 a_color;\n"
   "attribute vec4 a_texcoord;\n"
#endif
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_Position = u_transform * a_vertex;\n"
   "   v_color = a_color;\n"
   "   v_texcoord = a_texcoord;\n"
   "}\n  "
   ;

static const char *shader_texture_cube_map_fp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "precision mediump float;  // set default precision\n"
   "\n"
#endif // DX_GLES
   "uniform samplerCube u_sampler;\n"
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_FragColor = textureCube(u_sampler, v_texcoord.xyz) * v_color;\n"
   "}\n  "
   ; 


// ---------------------------------------------------------------------------- shader_texture_rectangle
static const char *shader_texture_rectangle_vp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "//precision mediump float;  // set default precision\n"
#endif // DX_GLES
   "\n"
   "uniform mat4 u_transform;\n"
   "\n"
#ifdef DX_GLES
   "attribute highp   vec4 a_vertex;\n"
   "attribute mediump vec4 a_color;\n"
   "attribute highp   vec4 a_texcoord;\n"
#else
   "attribute vec4 a_vertex;\n"
   "attribute vec4 a_color;\n"
   "attribute vec4 a_texcoord;\n"
#endif
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_Position = u_transform * a_vertex;\n"
   "   v_color = a_color;\n"
   "   v_texcoord = a_texcoord;\n"
   "}\n  "
   ;

static const char *shader_texture_rectangle_fp_src = 
#ifdef DX_GLES
   "#version 100\n"
   "\n"
   "precision mediump float;  // set default precision\n"
   "\n"
#endif // DX_GLES
   "uniform sampler2DRect u_sampler;\n"
   "\n"
   "varying vec4 v_color;\n"
   "varying vec4 v_texcoord;\n"
   "\n"
   "void main() {\n"
   "   gl_FragColor = texture2DRect(u_sampler, v_texcoord.xy) * v_color;\n"
   "}\n  "
   ; 
